@@ -16,27 +16,24 @@
  */
 package org.apache.wicket.ajax;
 
+import java.time.Duration;
 import java.util.List;
-
 import org.apache.wicket.Component;
 import org.apache.wicket.Page;
 import org.apache.wicket.WicketRuntimeException;
 import org.apache.wicket.ajax.attributes.AjaxAttributeName;
-import org.apache.wicket.ajax.attributes.AjaxCallListener;
 import org.apache.wicket.ajax.attributes.AjaxRequestAttributes;
 import org.apache.wicket.ajax.attributes.AjaxRequestAttributes.Method;
 import org.apache.wicket.ajax.attributes.CallbackParameter;
 import org.apache.wicket.ajax.attributes.IAjaxCallListener;
 import org.apache.wicket.ajax.attributes.ThrottlingSettings;
-import org.apache.wicket.ajax.json.JSONArray;
-import org.apache.wicket.ajax.json.JSONException;
-import org.apache.wicket.ajax.json.JSONObject;
-import org.apache.wicket.ajax.json.JsonFunction;
+import org.apache.wicket.ajax.json.JSONFunction;
 import org.apache.wicket.ajax.json.JsonUtils;
 import org.apache.wicket.behavior.AbstractAjaxBehavior;
 import org.apache.wicket.markup.head.IHeaderResponse;
 import org.apache.wicket.markup.head.JavaScriptHeaderItem;
 import org.apache.wicket.markup.html.IComponentAwareHeaderContributor;
+import org.apache.wicket.markup.html.form.Form;
 import org.apache.wicket.protocol.http.WebApplication;
 import org.apache.wicket.request.Url;
 import org.apache.wicket.request.cycle.RequestCycle;
@@ -44,7 +41,13 @@ import org.apache.wicket.request.resource.PackageResourceReference;
 import org.apache.wicket.request.resource.ResourceReference;
 import org.apache.wicket.resource.CoreLibrariesContributor;
 import org.apache.wicket.util.string.Strings;
-import org.apache.wicket.util.time.Duration;
+import com.github.openjson.JSONArray;
+import com.github.openjson.JSONException;
+import com.github.openjson.JSONObject;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+
+import jakarta.servlet.http.HttpServletRequest;
 
 /**
  * The base class for Wicket's default AJAX implementation.
@@ -56,23 +59,24 @@ import org.apache.wicket.util.time.Duration;
  */
 public abstract class AbstractDefaultAjaxBehavior extends AbstractAjaxBehavior
 {
-
 	private static final long serialVersionUID = 1L;
 
+	private static final Logger LOG = LoggerFactory.getLogger(AbstractDefaultAjaxBehavior.class);
+
 	/** reference to the default indicator gif file. */
 	public static final ResourceReference INDICATOR = new PackageResourceReference(
 		AbstractDefaultAjaxBehavior.class, "indicator.gif");
 
-	private static final String DYNAMIC_PARAMETER_FUNCTION_TEMPLATE = "function(attrs){%s}";
-	private static final String PRECONDITION_FUNCTION_TEMPLATE = "function(attrs){%s}";
-	private static final String COMPLETE_HANDLER_FUNCTION_TEMPLATE = "function(attrs, jqXHR, textStatus){%s}";
-	private static final String FAILURE_HANDLER_FUNCTION_TEMPLATE = "function(attrs, jqXHR, errorMessage, textStatus){%s}";
-	private static final String SUCCESS_HANDLER_FUNCTION_TEMPLATE = "function(attrs, jqXHR, data, textStatus){%s}";
-	private static final String AFTER_HANDLER_FUNCTION_TEMPLATE = "function(attrs){%s}";
-	private static final String BEFORE_SEND_HANDLER_FUNCTION_TEMPLATE = "function(attrs, jqXHR, settings){%s}";
-	private static final String BEFORE_HANDLER_FUNCTION_TEMPLATE = "function(attrs){%s}";
-	private static final String INIT_HANDLER_FUNCTION_TEMPLATE = "function(attrs){%s}";
-	private static final String DONE_HANDLER_FUNCTION_TEMPLATE = "function(attrs){%s}";
+	private static final String DYNAMIC_PARAMETER_FUNCTION_SIGNATURE = "function(attrs)";
+	private static final String PRECONDITION_FUNCTION_SIGNATURE = "function(attrs)";
+	private static final String COMPLETE_HANDLER_FUNCTION_SIGNATURE = "function(attrs, jqXHR, textStatus)";
+	private static final String FAILURE_HANDLER_FUNCTION_SIGNATURE = "function(attrs, jqXHR, errorMessage, textStatus)";
+	private static final String SUCCESS_HANDLER_FUNCTION_SIGNATURE = "function(attrs, jqXHR, data, textStatus)";
+	private static final String AFTER_HANDLER_FUNCTION_SIGNATURE = "function(attrs)";
+	private static final String BEFORE_SEND_HANDLER_FUNCTION_SIGNATURE = "function(attrs, jqXHR, settings)";
+	private static final String BEFORE_HANDLER_FUNCTION_SIGNATURE = "function(attrs)";
+	private static final String INIT_HANDLER_FUNCTION_SIGNATURE = "function(attrs)";
+	private static final String DONE_HANDLER_FUNCTION_SIGNATURE = "function(attrs)";
 
 	/**
 	 * Subclasses should call super.onBind()
@@ -82,7 +86,15 @@ public abstract class AbstractDefaultAjaxBehavior extends AbstractAjaxBehavior
 	@Override
 	protected void onBind()
 	{
-		getComponent().setOutputMarkupId(true);
+		final Component component = getComponent();
+		
+		component.setOutputMarkupId(true);
+		
+		if (getStatelessHint(component))
+		{
+			//generate behavior id
+			component.getBehaviorId(this);
+		}
 	}
 
 	/**
@@ -99,8 +111,8 @@ public abstract class AbstractDefaultAjaxBehavior extends AbstractAjaxBehavior
 		RequestCycle requestCycle = component.getRequestCycle();
 		Url baseUrl = requestCycle.getUrlRenderer().getBaseUrl();
 		CharSequence ajaxBaseUrl = Strings.escapeMarkup(baseUrl.toString());
-		response.render(JavaScriptHeaderItem.forScript("Wicket.Ajax.baseUrl=\"" + ajaxBaseUrl +
-			"\";", "wicket-ajax-base-url"));
+		response.render(JavaScriptHeaderItem.forScript("Wicket.Ajax.baseUrl=\"" + ajaxBaseUrl
+			+ "\";", "wicket-ajax-base-url"));
 
 		renderExtraHeaderContributors(component, response);
 	}
@@ -138,19 +150,27 @@ public abstract class AbstractDefaultAjaxBehavior extends AbstractAjaxBehavior
 	{
 		AjaxRequestAttributes attributes = new AjaxRequestAttributes();
 		WebApplication application = (WebApplication)getComponent().getApplication();
-		AjaxRequestTargetListenerCollection ajaxRequestTargetListeners = application.getAjaxRequestTargetListeners();
+		AjaxRequestTargetListenerCollection ajaxRequestTargetListeners = application
+			.getAjaxRequestTargetListeners();
 		for (AjaxRequestTarget.IListener listener : ajaxRequestTargetListeners)
 		{
-			if (listener instanceof AjaxRequestTarget.AbstractListener)
-			{
-				((AjaxRequestTarget.AbstractListener)listener).updateAjaxAttributes(attributes);
-			}
+			listener.updateAjaxAttributes(this, attributes);
 		}
-		updateAjaxAttributesBackwardCompatibility(attributes);
 		updateAjaxAttributes(attributes);
 		return attributes;
 	}
 
+	/**
+	 * This method decides whether to continue processing or to abort the Ajax request when the method
+	 * is different than the {@link AjaxRequestAttributes#getMethod()}'s method.
+	 *
+	 * @return response that can either abort or continue the processing of the Ajax request
+	 */
+	protected Form.MethodMismatchResponse onMethodMismatch()
+	{
+		return Form.MethodMismatchResponse.CONTINUE;
+	}
+
 	/**
 	 * Gives a chance to the specializations to modify the attributes.
 	 * 
@@ -161,26 +181,6 @@ public abstract class AbstractDefaultAjaxBehavior extends AbstractAjaxBehavior
 	{
 	}
 
-	/**
-	 * The code below handles backward compatibility.
-	 * 
-	 * @param attributes
-	 */
-	private void updateAjaxAttributesBackwardCompatibility(final AjaxRequestAttributes attributes)
-	{
-		AjaxCallListener backwardCompatibleAjaxCallListener = new AjaxCallListener();
-		backwardCompatibleAjaxCallListener.onSuccess(getSuccessScript());
-		backwardCompatibleAjaxCallListener.onFailure(getFailureScript());
-		backwardCompatibleAjaxCallListener.onPrecondition(getPreconditionScript());
-		attributes.getAjaxCallListeners().add(backwardCompatibleAjaxCallListener);
-
-		AjaxChannel channel = getChannel();
-		if (channel != null)
-		{
-			attributes.setChannel(channel);
-		}
-	}
-
 	/**
 	 * <pre>
 	 * 				{
@@ -222,24 +222,11 @@ public abstract class AbstractDefaultAjaxBehavior extends AbstractAjaxBehavior
 
 		try
 		{
-			String formId = attributes.getFormId();
-			if (Strings.isEmpty(formId) == false)
-			{
-				attributesJson.put(AjaxAttributeName.FORM_ID.jsonName(), formId);
-			}
 			attributesJson.put(AjaxAttributeName.URL.jsonName(), getCallbackUrl());
-
-			String[] eventNames = attributes.getEventNames();
-			if (eventNames.length == 1)
+			Method method = attributes.getMethod();
+			if (Method.POST == method)
 			{
-				attributesJson.put(AjaxAttributeName.EVENT_NAME.jsonName(), eventNames[0]);
-			}
-			else
-			{
-				for (String eventName : eventNames)
-				{
-					attributesJson.append(AjaxAttributeName.EVENT_NAME.jsonName(), eventName);
-				}
+				attributesJson.put(AjaxAttributeName.METHOD.jsonName(), method);
 			}
 
 			if (component instanceof Page == false)
@@ -248,6 +235,12 @@ public abstract class AbstractDefaultAjaxBehavior extends AbstractAjaxBehavior
 				attributesJson.put(AjaxAttributeName.MARKUP_ID.jsonName(), componentId);
 			}
 
+			String formId = attributes.getFormId();
+			if (Strings.isEmpty(formId) == false)
+			{
+				attributesJson.put(AjaxAttributeName.FORM_ID.jsonName(), formId);
+			}
+
 			if (attributes.isMultipart())
 			{
 				attributesJson.put(AjaxAttributeName.IS_MULTIPART.jsonName(), true);
@@ -263,7 +256,8 @@ public abstract class AbstractDefaultAjaxBehavior extends AbstractAjaxBehavior
 			CharSequence childSelector = attributes.getChildSelector();
 			if (Strings.isEmpty(childSelector) == false)
 			{
-				attributesJson.put(AjaxAttributeName.CHILD_SELECTOR.jsonName(), childSelector);
+				attributesJson.put(AjaxAttributeName.CHILD_SELECTOR.jsonName(),
+						childSelector);
 			}
 
 			if (attributes.isSerializeRecursively())
@@ -281,51 +275,50 @@ public abstract class AbstractDefaultAjaxBehavior extends AbstractAjaxBehavior
 			{
 				if (ajaxCallListener != null)
 				{
-					if (ajaxCallListener instanceof AjaxCallListener) {
-						CharSequence initHandler = ((AjaxCallListener) ajaxCallListener).getInitHandler(component);
-						appendListenerHandler(initHandler, attributesJson,
-							AjaxAttributeName.INIT_HANDLER.jsonName(),
-							INIT_HANDLER_FUNCTION_TEMPLATE);
-					}
-					CharSequence beforeHandler = ajaxCallListener.getBeforeHandler(component);
+					CharSequence initHandler = ajaxCallListener.getInitHandler(component);
+					appendListenerHandler(initHandler, attributesJson,
+						AjaxAttributeName.INIT_HANDLER.jsonName(),
+						INIT_HANDLER_FUNCTION_SIGNATURE);
+
+						CharSequence beforeHandler = ajaxCallListener.getBeforeHandler(component);
 					appendListenerHandler(beforeHandler, attributesJson,
 						AjaxAttributeName.BEFORE_HANDLER.jsonName(),
-						BEFORE_HANDLER_FUNCTION_TEMPLATE);
+						BEFORE_HANDLER_FUNCTION_SIGNATURE);
 
-					CharSequence beforeSendHandler = ajaxCallListener.getBeforeSendHandler(component);
+					CharSequence beforeSendHandler = ajaxCallListener
+						.getBeforeSendHandler(component);
 					appendListenerHandler(beforeSendHandler, attributesJson,
 						AjaxAttributeName.BEFORE_SEND_HANDLER.jsonName(),
-						BEFORE_SEND_HANDLER_FUNCTION_TEMPLATE);
+						BEFORE_SEND_HANDLER_FUNCTION_SIGNATURE);
 
 					CharSequence afterHandler = ajaxCallListener.getAfterHandler(component);
 					appendListenerHandler(afterHandler, attributesJson,
-						AjaxAttributeName.AFTER_HANDLER.jsonName(), AFTER_HANDLER_FUNCTION_TEMPLATE);
+						AjaxAttributeName.AFTER_HANDLER.jsonName(), AFTER_HANDLER_FUNCTION_SIGNATURE);
 
 					CharSequence successHandler = ajaxCallListener.getSuccessHandler(component);
 					appendListenerHandler(successHandler, attributesJson,
 						AjaxAttributeName.SUCCESS_HANDLER.jsonName(),
-						SUCCESS_HANDLER_FUNCTION_TEMPLATE);
+						SUCCESS_HANDLER_FUNCTION_SIGNATURE);
 
 					CharSequence failureHandler = ajaxCallListener.getFailureHandler(component);
 					appendListenerHandler(failureHandler, attributesJson,
 						AjaxAttributeName.FAILURE_HANDLER.jsonName(),
-						FAILURE_HANDLER_FUNCTION_TEMPLATE);
+						FAILURE_HANDLER_FUNCTION_SIGNATURE);
 
 					CharSequence completeHandler = ajaxCallListener.getCompleteHandler(component);
 					appendListenerHandler(completeHandler, attributesJson,
 						AjaxAttributeName.COMPLETE_HANDLER.jsonName(),
-						COMPLETE_HANDLER_FUNCTION_TEMPLATE);
+						COMPLETE_HANDLER_FUNCTION_SIGNATURE);
 
 					CharSequence precondition = ajaxCallListener.getPrecondition(component);
 					appendListenerHandler(precondition, attributesJson,
-						AjaxAttributeName.PRECONDITION.jsonName(), PRECONDITION_FUNCTION_TEMPLATE);
+						AjaxAttributeName.PRECONDITION.jsonName(), PRECONDITION_FUNCTION_SIGNATURE);
+
+					CharSequence doneHandler = ajaxCallListener.getDoneHandler(component);
+					appendListenerHandler(doneHandler, attributesJson,
+						AjaxAttributeName.DONE_HANDLER.jsonName(),
+						DONE_HANDLER_FUNCTION_SIGNATURE);
 
-					if (ajaxCallListener instanceof AjaxCallListener) {
-						CharSequence doneHandler = ((AjaxCallListener) ajaxCallListener).getDoneHandler(component);
-						appendListenerHandler(doneHandler, attributesJson,
-							AjaxAttributeName.DONE_HANDLER.jsonName(),
-							DONE_HANDLER_FUNCTION_TEMPLATE);
-					}
 				}
 			}
 
@@ -341,9 +334,7 @@ public abstract class AbstractDefaultAjaxBehavior extends AbstractAjaxBehavior
 			{
 				for (CharSequence dynamicExtraParameter : dynamicExtraParameters)
 				{
-					String func = String.format(DYNAMIC_PARAMETER_FUNCTION_TEMPLATE,
-						dynamicExtraParameter);
-					JsonFunction function = new JsonFunction(func);
+					JSONFunction function = getJsonFunction(DYNAMIC_PARAMETER_FUNCTION_SIGNATURE, dynamicExtraParameter);
 					attributesJson.append(AjaxAttributeName.DYNAMIC_PARAMETER_FUNCTION.jsonName(),
 						function);
 				}
@@ -354,28 +345,37 @@ public abstract class AbstractDefaultAjaxBehavior extends AbstractAjaxBehavior
 				attributesJson.put(AjaxAttributeName.IS_ASYNC.jsonName(), false);
 			}
 
+			String[] eventNames = attributes.getEventNames();
+			if (eventNames.length == 1)
+			{
+				attributesJson.put(AjaxAttributeName.EVENT_NAME.jsonName(), eventNames[0]);
+			}
+			else
+			{
+				for (String eventName : eventNames)
+				{
+					attributesJson.append(AjaxAttributeName.EVENT_NAME.jsonName(), eventName);
+				}
+			}
+
 			AjaxChannel channel = attributes.getChannel();
 			if (channel != null && channel.equals(AjaxChannel.DEFAULT) == false)
 			{
 				attributesJson.put(AjaxAttributeName.CHANNEL.jsonName(), channel);
 			}
 
-			if (attributes.isAllowDefault())
+			if (attributes.isPreventDefault())
 			{
-				attributesJson.put(AjaxAttributeName.IS_ALLOW_DEFAULT.jsonName(), true);
+				attributesJson.put(AjaxAttributeName.IS_PREVENT_DEFAULT.jsonName(), true);
 			}
 
-			Method method = attributes.getMethod();
-			if (Method.POST == method)
+			if (AjaxRequestAttributes.EventPropagation.STOP
+				.equals(attributes.getEventPropagation()))
 			{
-				attributesJson.put(AjaxAttributeName.METHOD.jsonName(), method);
+				attributesJson.put(AjaxAttributeName.EVENT_PROPAGATION.jsonName(), "stop");
 			}
-
-			if (AjaxRequestAttributes.EventPropagation.BUBBLE.equals(attributes.getEventPropagation()))
-			{
-				attributesJson.put(AjaxAttributeName.EVENT_PROPAGATION.jsonName(), "bubble");
-			}
-			else if (AjaxRequestAttributes.EventPropagation.STOP_IMMEDIATE.equals(attributes.getEventPropagation()))
+			else if (AjaxRequestAttributes.EventPropagation.STOP_IMMEDIATE.equals(attributes
+				.getEventPropagation()))
 			{
 				attributesJson.put(AjaxAttributeName.EVENT_PROPAGATION.jsonName(), "stopImmediate");
 			}
@@ -384,7 +384,7 @@ public abstract class AbstractDefaultAjaxBehavior extends AbstractAjaxBehavior
 			if (requestTimeout != null)
 			{
 				attributesJson.put(AjaxAttributeName.REQUEST_TIMEOUT.jsonName(),
-					requestTimeout.getMilliseconds());
+					requestTimeout.toMillis());
 			}
 
 			boolean wicketAjaxResponse = attributes.isWicketAjaxResponse();
@@ -403,10 +403,14 @@ public abstract class AbstractDefaultAjaxBehavior extends AbstractAjaxBehavior
 			if (throttlingSettings != null)
 			{
 				JSONObject throttlingSettingsJson = new JSONObject();
-				throttlingSettingsJson.put(AjaxAttributeName.THROTTLING_ID.jsonName(),
-					throttlingSettings.getId());
+				String throttleId = throttlingSettings.getId();
+				if (throttleId == null)
+				{
+					throttleId = component.getMarkupId();
+				}
+				throttlingSettingsJson.put(AjaxAttributeName.THROTTLING_ID.jsonName(), throttleId);
 				throttlingSettingsJson.put(AjaxAttributeName.THROTTLING_DELAY.jsonName(),
-					throttlingSettings.getDelay().getMilliseconds());
+					throttlingSettings.getDelay().toMillis());
 				if (throttlingSettings.getPostponeTimerOnUpdate())
 				{
 					throttlingSettingsJson.put(
@@ -428,24 +432,28 @@ public abstract class AbstractDefaultAjaxBehavior extends AbstractAjaxBehavior
 	}
 
 	private void appendListenerHandler(final CharSequence handler, final JSONObject attributesJson,
-		final String propertyName, final String functionTemplate) throws JSONException
+		final String propertyName, final String signature) throws JSONException
 	{
 		if (Strings.isEmpty(handler) == false)
 		{
-			final JsonFunction function;
-			if (handler instanceof JsonFunction)
+			final JSONFunction function;
+			if (handler instanceof JSONFunction)
 			{
-				function = (JsonFunction)handler;
+				function = (JSONFunction)handler;
 			}
 			else
 			{
-				String func = String.format(functionTemplate, handler);
-				function = new JsonFunction(func);
+				function = getJsonFunction(signature, handler);
 			}
 			attributesJson.append(propertyName, function);
 		}
 	}
 
+	private JSONFunction getJsonFunction(String signature, CharSequence body) {
+		String func = signature + "{" + body + "}";
+		return new JSONFunction(func);
+	}
+
 	/**
 	 * Gives a chance to modify the JSON attributesJson that is going to be used as attributes for
 	 * the Ajax call.
@@ -549,7 +557,7 @@ public abstract class AbstractDefaultAjaxBehavior extends AbstractAjaxBehavior
 				{
 					JSONObject object = new JSONObject();
 					object.put("name", curExtraParameter.getAjaxParameterName());
-					object.put("value", new JsonFunction(curExtraParameter.getAjaxParameterCode()));
+					object.put("value", new JSONFunction(curExtraParameter.getAjaxParameterCode()));
 					jsonArray.put(object);
 				}
 				catch (JSONException e)
@@ -559,56 +567,13 @@ public abstract class AbstractDefaultAjaxBehavior extends AbstractAjaxBehavior
 			}
 		}
 		sb.append("var params = ").append(jsonArray).append(";\n");
-		sb.append("attrs.")
-			.append(AjaxAttributeName.EXTRA_PARAMETERS)
-			.append(" = params.concat(attrs.")
-			.append(AjaxAttributeName.EXTRA_PARAMETERS)
-			.append(" || []);\n");
+		sb.append("attrs.").append(AjaxAttributeName.EXTRA_PARAMETERS)
+				.append(" = params.concat(attrs.")
+				.append(AjaxAttributeName.EXTRA_PARAMETERS).append(" || []);\n");
 		sb.append("Wicket.Ajax.ajax(attrs);\n");
 		return sb;
 	}
 
-	/**
-	 * @return an optional javascript expression that determines whether the request will actually
-	 *         execute (in form of return XXX;);
-	 * @deprecated Use {@link org.apache.wicket.ajax.attributes.AjaxRequestAttributes}
-	 */
-	@Deprecated
-	protected CharSequence getPreconditionScript()
-	{
-		return null;
-	}
-
-	/**
-	 * @return javascript that will run when the ajax call finishes with an error status
-	 */
-	@Deprecated
-	protected CharSequence getFailureScript()
-	{
-		return null;
-	}
-
-	/**
-	 * @return javascript that will run when the ajax call finishes successfully
-	 */
-	@Deprecated
-	protected CharSequence getSuccessScript()
-	{
-		return null;
-	}
-
-	/**
-	 * Provides an AjaxChannel for this Behavior.
-	 * 
-	 * @return an AjaxChannel - Defaults to null.
-	 * @deprecated Use {@link org.apache.wicket.ajax.attributes.AjaxRequestAttributes}
-	 */
-	@Deprecated
-	protected AjaxChannel getChannel()
-	{
-		return null;
-	}
-
 	/**
 	 * Finds the markup id of the indicator. The default search order is: component, behavior,
 	 * component's parent hierarchy.
@@ -639,12 +604,23 @@ public abstract class AbstractDefaultAjaxBehavior extends AbstractAjaxBehavior
 		return null;
 	}
 
-	/**
-	 * @see org.apache.wicket.behavior.IBehaviorListener#onRequest()
-	 */
 	@Override
 	public final void onRequest()
 	{
+		Form.MethodMismatchResponse methodMismatch = onMethodMismatch();
+		if (methodMismatch == Form.MethodMismatchResponse.ABORT)
+		{
+			AjaxRequestAttributes attrs = getAttributes();
+			String desiredMethod = attrs.getMethod().toString();
+			String actualMethod = ((HttpServletRequest) RequestCycle.get().getRequest().getContainerRequest()).getMethod();
+			if (!desiredMethod.equalsIgnoreCase(actualMethod))
+			{
+				LOG.debug("Ignoring the Ajax request because its method '{}' is different than the expected one '{}",
+				          actualMethod, desiredMethod);
+				return;
+			}
+		}
+
 		WebApplication app = (WebApplication)getComponent().getApplication();
 		AjaxRequestTarget target = app.newAjaxRequestTarget(getComponent().getPage());
 
